// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.subscriber;

import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ITestSubscription;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

@Repository
public class TestSubscriptionAzure implements ITestSubscription {
    @Autowired
    private ISubscriptionRepository subscriptionRepository;
    @Autowired
    private JaxRsDpsLog logger;

    @Override
    public void performTestAcknowledgement() throws Exception {
        try {
            /* Creation of Test Acknowledgement Subscription which ensures the successful delivery
            of notification at the test end point */
            Subscription subscription = new Subscription();
            subscription.setName("AcknowledgementTestSubscription");
            subscription.setDescription("Subscription for Ack of test-endpoint");
            subscription.setTopic("recordstopic");
            subscription.setPushEndpoint("http://TestAck/api/register/v1/test/challenge/1");
            subscription.setNotificationId("testingAcknowledgement");
            subscription.setId("AckSubscription");
            HmacSecret secret = new HmacSecret();
            secret.setValue("TestAcknowledgementSubscription");
            subscription.setSecret(secret);
            subscription = subscriptionRepository.create(subscription);
            logger.info("Successfully created Acknowledgement Subscription");

        }catch(Exception e)
        {
            logger.error("An exception occurred while creating Test Acknowledgment Subscription");
            throw e;
        }
    }
}
