package org.opengroup.osdu.register.provider.azure.subscriber.messageBus;

import com.microsoft.azure.servicebus.management.ManagementClient;
import com.microsoft.azure.servicebus.primitives.ConnectionStringBuilder;
import org.opengroup.osdu.azure.partition.PartitionInfoAzure;
import org.opengroup.osdu.azure.partition.PartitionServiceClient;
import org.opengroup.osdu.common.Validators;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
class ManagementClientFactoryImpl implements IManagementClientFactory {
    @Autowired
    private PartitionServiceClient partitionService;
    @Autowired
    private ManagementClientCache clientCache;
    @Autowired
    private DpsHeaders dpsHeaders;

    public ManagementClientFactoryImpl() {
    }

    public ManagementClient getManager() {
        Validators.checkNotNullAndNotEmpty(dpsHeaders.getPartitionId(), "partitionId");
        String cacheKey = dpsHeaders.getPartitionId() + "-managementClient";
        if (this.clientCache.containsKey(cacheKey)) {
            return (ManagementClient) this.clientCache.get(cacheKey);
        } else {
            PartitionInfoAzure pi = this.partitionService.getPartition(dpsHeaders.getPartitionId());
            String serviceBusConnectionString = pi.getSbConnection();
            ConnectionStringBuilder connectionStringBuilder = new ConnectionStringBuilder(serviceBusConnectionString);
            ManagementClient managementClient = new ManagementClient(connectionStringBuilder);
            this.clientCache.put(cacheKey, managementClient);
            return managementClient;
        }
    }
}

