package org.opengroup.osdu.register.provider.azure.util;

import java.nio.charset.StandardCharsets;
import java.util.Base64;

import org.opengroup.osdu.azure.cryptography.CryptographyClientFactory;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.azure.security.keyvault.keys.cryptography.CryptographyClient;
import com.azure.security.keyvault.keys.cryptography.models.DecryptResult;
import com.azure.security.keyvault.keys.cryptography.models.EncryptResult;
import com.azure.security.keyvault.keys.cryptography.models.EncryptionAlgorithm;

@Component
public class CryptographyUtil {

  @Autowired
  private CryptographyClientFactory factory;
  @Autowired
  private PartitionHelper partitionHelper;
  @Autowired
  private DpsHeaders headers;

  public String encryptData(String plainText) {
    CryptographyClient client = factory.getClient(headers.getPartitionId());
    EncryptResult encryptResult = client.encrypt(EncryptionAlgorithm.RSA_OAEP, plainText.getBytes());
    byte[] val = encryptResult.getCipherText();
    return Base64.getEncoder().encodeToString(val);
  }

  public String decryptData(String cipherText) {
    CryptographyClient client = factory.getClient(headers.getPartitionId());
    DecryptResult decryptResult = client.decrypt(EncryptionAlgorithm.RSA_OAEP, Base64.getDecoder().decode(cipherText));
    return new String(decryptResult.getPlainText(), StandardCharsets.UTF_8);
  }

  public String getKeyName() {
    String keyIdentifier = partitionHelper.retrieveKeyIdentifier();
    String[] stringArray = keyIdentifier.split("/");
    return stringArray[stringArray.length - 2];
  }

  public String getKeyVersion() {
    String keyIdentifier = partitionHelper.retrieveKeyIdentifier();
    String[] stringArray = keyIdentifier.split("/");
    return stringArray[stringArray.length - 1];
  }

}
