// Copyright 2017-2021, Schlumberger
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.util;

import org.apache.http.HttpStatus;
import org.opengroup.osdu.azure.partition.EventGridTopicPartitionInfoAzure;
import org.opengroup.osdu.azure.partition.PartitionInfoAzure;
import org.opengroup.osdu.azure.partition.PartitionServiceClient;
import org.opengroup.osdu.azure.partition.PartitionServiceEventGridClient;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.partition.PartitionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

@Component
public class PartitionHelper {

    @Autowired
    private PartitionServiceClient partitionService;
    @Autowired
    private PartitionServiceEventGridClient partitionServiceEventGridClient;
    @Autowired
    private DpsHeaders headers;

    public String retrieveResourceGroup() {
        PartitionInfoAzure partition = partitionService.getPartition(headers.getPartitionId());
        return partition.getEventGridResourceGroup();
    }

    public String getFullyQualifiedTopicName(String topicNameAlias) {
        EventGridTopicPartitionInfoAzure eventGridTopicPartitionInfoAzure = null;
        try {
            eventGridTopicPartitionInfoAzure = partitionServiceEventGridClient.getEventGridTopicInPartition(headers.getPartitionId(), topicNameAlias);
        } catch (PartitionException e) {
          throw new AppException(
                  HttpStatus.SC_NOT_FOUND, "Not found", String.format("Invalid topic for subscription: %s, please use GET topics API for available topics", topicNameAlias), e);
        }
        return eventGridTopicPartitionInfoAzure.getTopicName().split("\\.")[0].replace("https://", "");
    }


    public String retrieveKeyIdentifier() {
        PartitionInfoAzure partition = partitionService.getPartition(headers.getPartitionId());
        return partition.getCryptographyEncryptionKeyIdentifier();
    }

    public String retrieveScope(String topic) {
        PartitionInfoAzure partition = partitionService.getPartition(headers.getPartitionId());
        return String.format("/subscriptions/%s/resourceGroups/%s/providers/Microsoft.EventGrid/topics/%s",
                partition.getAzureSubscriptionId(), partition.getEventGridResourceGroup(), topic);
    }
}
