/*
 * Copyright 2017-2020, Schlumberger
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.register.provider.gcp.ddms.datastore;

import static java.util.Objects.isNull;

import com.google.cloud.datastore.Datastore;
import java.util.HashMap;
import java.util.Map;
import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.core.gcp.multitenancy.IDatastoreFactory;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
public class DatastoreMultiTenantAccess {

    private final IDatastoreFactory factory;

    private final Map<String, Datastore> tenantRepositories = new HashMap<>();

    public Datastore get(TenantInfo tenantInfo) {
        if (isNull(tenantInfo)){
            throw invalidTenantGivenException("null");
        }
        if (!tenantRepositories.containsKey(tenantInfo.getName())) {
            addRepository(tenantInfo);
        }
        return tenantRepositories.get(tenantInfo.getName());
    }

    private void addRepository(TenantInfo tenantInfo) {
        Datastore ds = factory.getDatastore(tenantInfo);
        if (ds == null) {
            throw invalidTenantGivenException(tenantInfo.getName());
        }
        tenantRepositories.put(tenantInfo.getName(), ds);
    }

    private AppException invalidTenantGivenException(String tenantName) {
        return new AppException(403, "Forbidden", String.format("You do not have access to the %s value given %s",
                DpsHeaders.DATA_PARTITION_ID, tenantName));
    }
}
