/* Licensed Materials - Property of IBM              */		
/* (c) Copyright IBM Corp. 2020. All Rights Reserved.*/	


package org.opengroup.osdu.register.provider.ibm.ddms.datastore;

import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.apache.commons.lang3.StringUtils;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.ibm.auth.ServiceCredentials;
import org.opengroup.osdu.core.ibm.cloudant.IBMCloudantClientFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.cloudant.client.api.Database;

import lombok.extern.slf4j.Slf4j;
@Slf4j
@Component
public class DatastoreMultiTenantAccess {
	@Value("${ibm.db.url}")
	private String dbUrl;
	
	@Value("${ibm.env.prefix}")
	private String dBEnvPrefix;
	
	@Value("${ibm.db.user:#{null}}")
	private String dbUser;
	
	@Value("${ibm.db.password:#{null}}")
	private String dbPassword;

    //private final IDatastoreFactory factory;
    
    private IBMCloudantClientFactory cloudantFactory;

    private final Map<String, Database> tenantRepositories = new HashMap<>();

   /* public DatastoreMultiTenantAccess() {
        this(new IBMCloudantClientFactory(new ServiceCredentials(dbUrl, dbUser, dbPassword)));
    }

    DatastoreMultiTenantAccess(IBMCloudantClientFactory factory) {
        this.cloudantFactory = factory;
    }*/
    
    @PostConstruct
	public void init() {
		cloudantFactory = new IBMCloudantClientFactory(new ServiceCredentials(dbUrl, dbUser, dbPassword));
		log.info("IBM Cloudant factory created ");
		
    }

    public Database get(String tenantName) {
        if (StringUtils.isBlank(tenantName))
            throw invalidTenantGivenException(tenantName);
        if (!tenantRepositories.containsKey(tenantName)) {
            addRepository(tenantName);
        }
        return tenantRepositories.get(tenantName);
    }

    private void addRepository(String tenantName) {
        
		try {
			Database db = cloudantFactory.getDatabase(dBEnvPrefix, tenantName);
			if (db == null)
	            throw invalidTenantGivenException(tenantName);
	        tenantRepositories.put(tenantName, db);
	        log.info("Database created and added to tenant repository : "+db.info().getDbName());
		} catch (MalformedURLException e) {
			log.error("MalformedURLException while creating Database ");
			e.printStackTrace();
		}
        
    }

    private AppException invalidTenantGivenException(String tenantName) {
        return new AppException(403, "Forbidden", String.format("You do not have access to the %s value given %s",
                DpsHeaders.DATA_PARTITION_ID, tenantName));
    }
}
